@@ -8,8 +8,8 @@ import ast
 from cxxgen import *
 from cxxtypes import *
 
-from analysis import LocalDeclarations, GlobalDeclarations, Scope
-from analysis import YieldPoints, BoundedExpressions, ArgumentEffects
+from analyses import LocalDeclarations, GlobalDeclarations, Scope, Dependencies
+from analyses import YieldPoints, BoundedExpressions, ArgumentEffects
 from passmanager import Backend
 
 from tables import operator_to_lambda, modules, type_to_suffix
@@ -20,7 +20,7 @@ from syntax import PythranSyntaxError
 
 from openmp import OMPDirective
 
-from math import isnan
+from math import isnan, isinf
 
 import cStringIO
 import unparse
@@ -53,10 +53,9 @@ def templatize(node, types, default_types=None):
         default_types = [None] * len(types)
     if types:
         return Template(
-                ["typename {0} {1}".format(
-                    t, "= {0}".format(d) if d else "")
-                    for t, d in zip(types, default_types)],
-                node)
+            ["typename {0} {1}".format(t, "= {0}".format(d) if d else "")
+             for t, d in zip(types, default_types)],
+            node)
     else:
         return node
 
@@ -77,10 +76,11 @@ class Cxx(Backend):
     >>> pm = passmanager.PassManager('test')
     >>> r = pm.dump(Cxx, node)
     >>> print r
-    #include <pythran/pythran.h>
+    #include <pythonic/__builtin__/print.hpp>
+    #include <pythonic/__builtin__/str.hpp>
     namespace __pythran_test
     {
-      print(core::string("hello world"));
+      pythonic::__builtin__::print(pythonic::types::str("hello world"));
     }
     '''
 
@@ -95,14 +95,16 @@ class Cxx(Backend):
         self.definitions = list()
         self.break_handlers = list()
         self.result = None
-        super(Cxx, self).__init__(
-                GlobalDeclarations, BoundedExpressions, Types, ArgumentEffects,
-                Scope)
+        super(Cxx, self).__init__(Dependencies, GlobalDeclarations,
+                                  BoundedExpressions, Types, ArgumentEffects,
+                                  Scope)
 
     # mod
     def visit_Module(self, node):
         # build all types
-        header = Include("pythran/pythran.h")
+        def gen_include(t):
+            return "/".join(("pythonic",) + t) + ".hpp"
+        headers = map(Include, sorted(map(gen_include, self.dependencies)))
 
         # remove top-level strings
         fbody = (n for n in node.body if not isinstance(n, ast.Expr))
@@ -110,7 +112,7 @@ class Cxx(Backend):
 
         nsbody = body + self.declarations + self.definitions
         ns = Namespace(pythran_ward + self.passmanager.module_name, nsbody)
-        self.result = CompilationUnit([header, ns])
+        self.result = CompilationUnit(headers + [ns])
 
     # local declaration processing
     def process_locals(self, node, node_visited, *skipped):
@@ -138,9 +140,11 @@ class Cxx(Backend):
                     hasfor = 'for' in directive.s
                     nodefault = 'default' not in directive.s
                     noindexref = all(x.id != target.id for x in directive.deps)
-                    if hasfor and nodefault and noindexref:
+                    if (hasfor and nodefault and noindexref and
+                            target.id not in self.scope[node]):
                         directive.s += ' private({})'
-                        directive.deps.append(ast.Name(target.id, ast.Param()))
+                        directive.deps.append(ast.Name(target.id, ast.Load()))
+                directive.deps = map(self.visit, directive.deps)
                 directives.append(directive)
             if index is None:
                 stmt = AnnotatedStatement(stmt, directives)
@@ -179,7 +183,7 @@ class Cxx(Backend):
                         self.mapping[node] = len(self.mapping)
                         self.cache[node] = t
                 return CachedTypeVisitor.CachedType(
-                        "__type{0}".format(self.mapping[node]))
+                    "__type{0}".format(self.mapping[node]))
 
             def typedefs(self):
                 l = sorted(self.mapping.items(), key=lambda x: x[1])
@@ -213,20 +217,19 @@ class Cxx(Backend):
                        {n.id: n for n in self.ldecls}.iteritems()}
 
         # 0 is used as initial_state, thus the +1
-        self.yields = {k: (1 + v, "yield_point{0}".format(1 + v))
-                for (v, k) in
-                enumerate(self.passmanager.gather(YieldPoints, node))}
+        self.yields = {k: (1 + v, "yield_point{0}".format(1 + v)) for (v, k) in
+                       enumerate(self.passmanager.gather(YieldPoints, node))}
 
         # gather body dump
         operator_body = map(self.visit, node.body)
 
         # compute arg dump
         default_arg_values = (
-                [None] * (len(node.args.args) - len(node.args.defaults))
-                + [self.visit(n) for n in node.args.defaults])
+            [None] * (len(node.args.args) - len(node.args.defaults))
+            + [self.visit(n) for n in node.args.defaults])
         default_arg_types = (
-                [None] * (len(node.args.args) - len(node.args.defaults))
-                + [self.types[n] for n in node.args.defaults])
+            [None] * (len(node.args.args) - len(node.args.defaults))
+            + [self.types[n] for n in node.args.defaults])
 
         # compute type dump
         result_type = self.types[node][0]
@@ -234,7 +237,7 @@ class Cxx(Backend):
         callable_type = Typedef(Value("void", "callable"))
 
         def make_function_declaration(rtype, name, ftypes, fargs,
-            defaults=None, attributes=[]):
+                                      defaults=None, attributes=[]):
             if defaults is None:
                 defaults = [None] * len(ftypes)
             arguments = list()
@@ -245,16 +248,17 @@ class Cxx(Backend):
                     rvalue_ref = "&&"
                 else:
                     rvalue_ref = " const &"
-                argument = Value(t + rvalue_ref,
+                argument = Value(
+                    t + rvalue_ref,
                     "{0}{1}".format(a, "= {0}".format(d) if d else ""))
                 arguments.append(argument)
             return FunctionDeclaration(Value(rtype, name), arguments,
-                *attributes)
+                                       *attributes)
 
         def make_const_function_declaration(rtype, name, ftypes, fargs,
-            defaults=None):
+                                            defaults=None):
             return make_function_declaration(rtype, name, ftypes, fargs,
-                defaults, ["const"])
+                                             defaults, ["const"])
 
         if self.yields:  # generator case
             # a generator has a call operator that returns the iterator
@@ -275,19 +279,18 @@ class Cxx(Backend):
 
             # the constructors
             next_constructors = [
-                    FunctionBody(
-                        FunctionDeclaration(Value("", next_name), []),
-                        Line(': {}(0) {{}}'.format(Cxx.generator_state_holder))
-                        )]
+                FunctionBody(
+                    FunctionDeclaration(Value("", next_name), []),
+                    Line(': {}(0) {{}}'.format(Cxx.generator_state_holder))
+                    )]
             if formal_types:
                 #if all parameters have a default value, we don't need default
                 # constructor
                 if default_arg_values and all(default_arg_values):
                     next_constructors = list()
                 next_constructors.append(FunctionBody(
-                    make_function_declaration("",
-                        next_name, formal_types, formal_args,
-                        default_arg_values),
+                    make_function_declaration("", next_name, formal_types,
+                                              formal_args, default_arg_values),
                     Line("{0} {{ }}".format(
                         ": {0}".format(
                             ", ".join(
@@ -298,152 +301,145 @@ class Cxx(Backend):
                     ))
 
             next_iterator = [
-                    FunctionBody(
-                        FunctionDeclaration(Value("void", "operator++"), []),
-                        Block([Statement("next()")])),
-
-                    FunctionBody(
-                        FunctionDeclaration(
-                            Value("typename {0}::result_type".format(
-                                instanciated_next_name),
-                                "operator*"),
-                            [], "const"),
-                        Block([
-                            ReturnStatement(
-                                Cxx.generator_state_value)])),
-                    FunctionBody(
-                        FunctionDeclaration(
-                            Value("pythonic::generator_iterator<{0}>".format(
-                                next_name),
-                                "begin"),
-                            []),
-                        Block([Statement("next()"),
-                            ReturnStatement(
-                                "generator_iterator<{0}>(*this)".format(
-                                    next_name))])),
-                    FunctionBody(
-                        FunctionDeclaration(
-                            Value("pythonic::generator_iterator<{0}>".format(
-                                next_name),
-                                "end"),
-                            []),
-                        Block([ReturnStatement(
-                            "generator_iterator<{0}>()".format(next_name))])),
-                    FunctionBody(
-                        FunctionDeclaration(
-                            Value("bool", "operator!="),
-                            [Value("{0} const &".format(next_name), "other")],
-                            "const"),
-                        Block([ReturnStatement(
-                            "{0}!=other.{0}".format(
-                                Cxx.generator_state_holder))])),
-                    FunctionBody(
-                        FunctionDeclaration(
-                            Value("bool", "operator=="),
-                            [Value("{0} const &".format(next_name), "other")],
-                            "const"),
-                        Block([ReturnStatement(
-                            "{0}==other.{0}".format(
-                                Cxx.generator_state_holder))])),
-                    ]
-            next_signature = templatize(
+                FunctionBody(
+                    FunctionDeclaration(Value("void", "operator++"), []),
+                    Block([Statement("next()")])),
+                FunctionBody(
                     FunctionDeclaration(
-                        Value(
-                            "typename {0}::result_type".format(
-                                instanciated_next_name),
-                            "{0}::next".format(instanciated_next_name)),
+                        Value("typename {0}::result_type".format(
+                            instanciated_next_name),
+                            "operator*"),
+                        [], "const"),
+                    Block([
+                        ReturnStatement(
+                            Cxx.generator_state_value)])),
+                FunctionBody(
+                    FunctionDeclaration(
+                        Value("pythonic::types::generator_iterator<{0}>"
+                              .format(next_name),
+                              "begin"),
                         []),
-                    formal_types)
+                    Block([Statement("next()"),
+                           ReturnStatement(
+                               "pythonic::types::generator_iterator<{0}>"
+                               "(*this)".format(next_name))])),
+                FunctionBody(
+                    FunctionDeclaration(
+                        Value("pythonic::types::generator_iterator<{0}>"
+                              .format(next_name),
+                              "end"),
+                        []),
+                    Block([ReturnStatement(
+                        "pythonic::types::generator_iterator<{0}>()"
+                        .format(next_name))])),
+                FunctionBody(
+                    FunctionDeclaration(
+                        Value("bool", "operator!="),
+                        [Value("{0} const &".format(next_name), "other")],
+                        "const"),
+                    Block([ReturnStatement(
+                        "{0}!=other.{0}".format(
+                            Cxx.generator_state_holder))])),
+                FunctionBody(
+                    FunctionDeclaration(
+                        Value("bool", "operator=="),
+                        [Value("{0} const &".format(next_name), "other")],
+                        "const"),
+                    Block([ReturnStatement(
+                        "{0}==other.{0}".format(
+                            Cxx.generator_state_holder))])),
+                ]
+            next_signature = templatize(
+                FunctionDeclaration(
+                    Value(
+                        "typename {0}::result_type".format(
+                            instanciated_next_name),
+                        "{0}::next".format(instanciated_next_name)),
+                    []),
+                formal_types)
 
             next_body = operator_body
             # the dispatch table at the entry point
-            next_body.insert(0,
-                    Statement("switch({0}) {{ {1} }}".format(
-                        Cxx.generator_state_holder,
-                        " ".join("case {0}: goto {1};".format(num, where)
-                            for (num, where) in sorted(
-                                self.yields.itervalues(),
-                                key=lambda x: x[0])))))
+            next_body.insert(0, Statement("switch({0}) {{ {1} }}".format(
+                Cxx.generator_state_holder,
+                " ".join("case {0}: goto {1};".format(num, where)
+                         for (num, where) in sorted(
+                             self.yields.itervalues(),
+                             key=lambda x: x[0])))))
 
             ctx = CachedTypeVisitor(lctx)
-            next_members = (
-                    [Statement("{0} {1}".format(ft, fa))
-                        for (ft, fa) in zip(formal_types, formal_args)]
-                    + [Statement("{0} {1}".format(
-                        self.types[k].generate(ctx),
-                        k.id))
-                        for k in self.ldecls]
-                    + [Statement("{0} {1}".format(v, k))
-                        for k, v in self.extra_declarations]
-                    + [Statement("{0} {1}".format("long",
-                        Cxx.generator_state_holder))]
-                    + [Statement("typename {0}::result_type {1}".format(
-                        instanciated_next_name,
-                        Cxx.generator_state_value))]
-                    )
+            next_members = ([Statement("{0} {1}".format(ft, fa))
+                             for (ft, fa) in zip(formal_types, formal_args)]
+                            + [Statement(
+                                "{0} {1}".format(self.types[k].generate(ctx),
+                                                 k.id))
+                               for k in self.ldecls]
+                            + [Statement("{0} {1}".format(v, k))
+                               for k, v in self.extra_declarations]
+                            + [Statement("{0} {1}".format("long",
+                               Cxx.generator_state_holder))]
+                            + [Statement(
+                                "typename {0}::result_type {1}".format(
+                                    instanciated_next_name,
+                                    Cxx.generator_state_value))])
 
             extern_typedefs = [Typedef(Value(t.generate(ctx), t.name))
-                    for t in self.types[node][1] if not t.isweak()]
+                               for t in self.types[node][1] if not t.isweak()]
             iterator_typedef = [
-                    Typedef(Value(
-                        "pythonic::generator_iterator<{0}>".format(
-                            "{0}<{1}>".format(
-                                next_name,
-                                ", ".join(t for t in formal_types))
-                            if formal_types
-                            else next_name),
+                Typedef(
+                    Value("pythonic::types::generator_iterator<{0}>".format(
+                        "{0}<{1}>".format(next_name, ", ".join(formal_types))
+                        if formal_types else next_name),
                         "iterator")),
-                    Typedef(Value(
-                        result_type.generate(ctx),
-                        "value_type"))]
+                Typedef(Value(result_type.generate(ctx),
+                              "value_type"))]
             result_typedef = [
-                    Typedef(Value(result_type.generate(ctx), "result_type"))]
+                Typedef(Value(result_type.generate(ctx), "result_type"))]
             extra_typedefs = (ctx.typedefs()
-                    + extern_typedefs
-                    + iterator_typedef
-                    + result_typedef)
+                              + extern_typedefs
+                              + iterator_typedef
+                              + result_typedef)
 
             next_struct = templatize(
-                    Struct(
-                        next_name,
-                        extra_typedefs
-                        + next_members
-                        + next_constructors
-                        + next_iterator
-                        + next_declaration),
-                    formal_types)
+                Struct(next_name,
+                       extra_typedefs
+                       + next_members
+                       + next_constructors
+                       + next_iterator
+                       + next_declaration),
+                formal_types)
             next_definition = FunctionBody(next_signature, Block(next_body))
 
             operator_declaration = [
-                    templatize(
-                        make_const_function_declaration(
-                            instanciated_next_name,
-                            "operator()",
-                            formal_types,
-                            formal_args,
-                            default_arg_values),
-                        formal_types,
-                        default_arg_types),
-                    EmptyStatement()]
-            operator_signature = make_const_function_declaration(
-                    instanciated_next_name,
-                    "{0}::operator()".format(node.name),
-                    formal_types,
-                    formal_args)
-            operator_definition = FunctionBody(
-                    templatize(operator_signature, formal_types),
-                    Block([ReturnStatement("{0}({1})".format(
+                templatize(
+                    make_const_function_declaration(
                         instanciated_next_name,
-                        ", ".join(formal_args)))])
-                    )
+                        "operator()",
+                        formal_types,
+                        formal_args,
+                        default_arg_values),
+                    formal_types,
+                    default_arg_types),
+                EmptyStatement()]
+            operator_signature = make_const_function_declaration(
+                instanciated_next_name,
+                "{0}::operator()".format(node.name),
+                formal_types,
+                formal_args)
+            operator_definition = FunctionBody(
+                templatize(operator_signature, formal_types),
+                Block([ReturnStatement("{0}({1})".format(
+                    instanciated_next_name,
+                    ", ".join(formal_args)))])
+                )
 
             topstruct_type = templatize(
-                    Struct("type", extra_typedefs),
-                    formal_types)
+                Struct("type", extra_typedefs),
+                formal_types)
             topstruct = Struct(
-                    node.name,
-                    [topstruct_type, callable_type]
-                    + operator_declaration)
+                node.name,
+                [topstruct_type, callable_type]
+                + operator_declaration)
 
             self.declarations.append(next_struct)
             self.definitions.append(next_definition)
@@ -451,66 +447,63 @@ class Cxx(Backend):
         else:  # regular function case
             # a function has a call operator to be called
             # and a default constructor to create instances
-            fscope = "type{0}::".format(
-                    "<{0}>".format(
-                        ", ".join(formal_types))
-                    if formal_types
-                    else "")
+            fscope = "type{0}::".format("<{0}>".format(", ".join(formal_types))
+                                        if formal_types
+                                        else "")
             ffscope = "{0}::{1}".format(node.name, fscope)
 
             operator_declaration = [
-                    templatize(
-                        make_const_function_declaration(
-                            "typename {0}result_type".format(fscope),
-                            "operator()",
-                            formal_types,
-                            formal_args,
-                            default_arg_values),
+                templatize(
+                    make_const_function_declaration(
+                        "typename {0}result_type".format(fscope),
+                        "operator()",
                         formal_types,
-                        default_arg_types),
-                    EmptyStatement()
-                    ]
-            operator_signature = make_const_function_declaration(
-                    "typename {0}result_type".format(ffscope),
-                    "{0}::operator()".format(node.name),
+                        formal_args,
+                        default_arg_values),
                     formal_types,
-                    formal_args)
+                    default_arg_types),
+                EmptyStatement()
+                ]
+            operator_signature = make_const_function_declaration(
+                "typename {0}result_type".format(ffscope),
+                "{0}::operator()".format(node.name),
+                formal_types,
+                formal_args)
             ctx = CachedTypeVisitor(lctx)
             operator_local_declarations = (
-                    [Statement("{0} {1}".format(
-                     self.types[k].generate(ctx), k.id)) for k in self.ldecls]
-                    + [Statement("{0} {1}".format(v, k))
-                       for k, v in self.extra_declarations]
-                    )
+                [Statement("{0} {1}".format(
+                 self.types[k].generate(ctx), k.id)) for k in self.ldecls]
+                + [Statement("{0} {1}".format(v, k))
+                   for k, v in self.extra_declarations]
+                )
             dependent_typedefs = ctx.typedefs()
             operator_definition = FunctionBody(
-                    templatize(operator_signature, formal_types),
-                    Block(dependent_typedefs
-                        + operator_local_declarations
-                        + operator_body)
-                    )
+                templatize(operator_signature, formal_types),
+                Block(dependent_typedefs
+                      + operator_local_declarations
+                      + operator_body)
+                )
 
             ctx = CachedTypeVisitor()
             extra_typedefs = (
-                    [Typedef(Value(t.generate(ctx), t.name))
-                        for t in self.types[node][1] if not t.isweak()]
-                    + [Typedef(Value(
-                        result_type.generate(ctx),
-                        "result_type"))]
-                    )
+                [Typedef(Value(t.generate(ctx), t.name))
+                 for t in self.types[node][1] if not t.isweak()]
+                + [Typedef(Value(
+                    result_type.generate(ctx),
+                    "result_type"))]
+                )
             extra_typedefs = ctx.typedefs() + extra_typedefs
             return_declaration = [
-                    templatize(
-                        Struct("type", extra_typedefs),
-                        formal_types,
-                        default_arg_types
-                        )
-                    ]
-            topstruct = Struct(
-                    node.name,
-                    [callable_type]
-                    + return_declaration
-                    + operator_declaration)
+                templatize(
+                    Struct("type", extra_typedefs),
+                    formal_types,
+                    default_arg_types
+                    )
+                ]
+            topstruct = Struct(node.name,
+                               [callable_type]
+                               + return_declaration
+                               + operator_declaration)
 
         self.declarations.append(topstruct)
         self.definitions.append(operator_definition)
@@ -544,10 +537,10 @@ class Cxx(Backend):
 
     def visit_Assign(self, node):
         if not all(isinstance(n, ast.Name) or isinstance(n, ast.Subscript)
-                for n in node.targets):
+                   for n in node.targets):
             raise PythranSyntaxError(
-                    "Must assign to an identifier or a subscript",
-                    node)
+                "Must assign to an identifier or a subscript",
+                node)
         value = self.visit(node.value)
         targets = [self.visit(t) for t in node.targets]
         alltargets = "= ".join(targets)
@@ -569,7 +562,7 @@ class Cxx(Backend):
         value = self.visit(node.value)
         target = self.visit(node.target)
         l = operator_to_lambda[type(node.op)]
-        if type(node.op) in (ast.FloorDiv,):
+        if type(node.op) in (ast.FloorDiv, ast.Mod, ast.Pow):
             stmt = Assign(target, l(target, value))
         else:
             stmt = Statement(l(target, '')[1:-2] + '= {0}'.format(value))
@@ -577,17 +570,17 @@ class Cxx(Backend):
 
     def visit_Print(self, node):
         values = [self.visit(n) for n in node.values]
-        stmt = Statement("print{0}({1})".format(
-                "" if node.nl else "_nonl",
-                ", ".join(values))
-                )
+        stmt = Statement("pythonic::__builtin__::print{0}({1})".format(
+            "" if node.nl else "_nonl",
+            ", ".join(values))
+            )
         return self.process_omp_attachements(node, stmt)
 
     def visit_For(self, node):
         if not isinstance(node.target, ast.Name):
             raise PythranSyntaxError(
-                    "Using something other than an identifier as loop target",
-                    node.target)
+                "Using something other than an identifier as loop target",
+                node.target)
         iter = self.visit(node.iter)
         target = self.visit(node.target)
 
@@ -626,50 +619,44 @@ class Cxx(Backend):
 
         auto_for = bool(metadata.get(node.target, metadata.LocalVariable))
         auto_for |= (type(node.target) is ast.Name
-                      and node.target.id in self.scope[node])
+                     and node.target.id in self.scope[node])
         auto_for &= not self.yields and not omp
 
         loop_body = self.process_locals(node, loop_body, node.target.id)
 
         if auto_for:
             self.ldecls = {d for d in self.ldecls if d.id != node.target.id}
-            loop = AutoFor(
-                    target,
-                    local_iter,
-                    loop_body
-                    )
+            loop = AutoFor(target, local_iter, loop_body)
         else:
-            if (metadata.get(node.target, metadata.LocalVariable) and
-                    not self.yields):
+            if node.target.id in self.scope[node] and not self.yields:
+                self.ldecls = {d for d in self.ldecls
+                               if d.id != node.target.id}
                 local_type = "typename decltype({})::reference ".format(
-                                local_target)
+                    local_target)
             else:
                 local_type = ""
-            loop_body_prelude = Statement(
-                    "{} {}= *{}".format(
-                        local_type,
-                        target,
-                        local_target)
-                    )
+            loop_body_prelude = Statement("{} {}= *{}".format(local_type,
+                                                              target,
+                                                              local_target))
             loop = For(
-                    "{0} {1} = {2}.begin()".format(
-                        local_target_decl,
-                        local_target,
-                        local_iter),
-                    "{0} < {1}.end()".format(
-                        local_target,
-                        local_iter),
-                    "++{0}".format(local_target),
-                    Block([loop_body_prelude, loop_body])
-                    )
+                "{0} {1} = {2}.begin()".format(
+                    local_target_decl,
+                    local_target,
+                    local_iter),
+                "{0} < {1}.end()".format(
+                    local_target,
+                    local_iter),
+                "++{0}".format(local_target),
+                Block([loop_body_prelude, loop_body])
+                )
         stmts = [prelude, loop]
 
         # in that case when can proceed to a reserve
         for comp in metadata.get(node, metadata.Comprehension):
             stmts.insert(1,
-                    Statement("pythonic::reserve({0},{1})".format(
-                        comp.target,
-                        local_iter)))
+                         Statement("pythonic::utils::reserve({0},{1})".format(
+                             comp.target,
+                             local_iter)))
 
         if break_handler:
             orelse = map(self.visit, node.orelse)
@@ -746,7 +733,7 @@ class Cxx(Backend):
     def visit_Assert(self, node):
         params = [self.visit(node.test), node.msg and self.visit(node.msg)]
         sparams = ", ".join(map(strip_exp, filter(None, params)))
-        return Statement("pythran_assert({0})".format(sparams))
+        return Statement("pythonic::pythran_assert({0})".format(sparams))
 
     def visit_Import(self, node):
         return EmptyStatement()  # everything is already #included
@@ -805,43 +792,43 @@ class Cxx(Backend):
 
     def visit_List(self, node):
         if not node.elts:  # empty list
-            return "__builtin__::list()"
+            return "pythonic::__builtin__::list()"
         else:
             elts = [self.visit(n) for n in node.elts]
             # constructor disambiguation, clang++ workaround
             if len(elts) == 1:
-                elts.append('core::single_value()')
+                elts.append('pythonic::types::single_value()')
                 return "{0}({{ {1} }})".format(
-                        Assignable(self.types[node]),
-                        ", ".join(elts))
+                    Assignable(self.types[node]),
+                    ", ".join(elts))
             else:
                 return "{0}({{ {1} }})".format(
-                        Assignable(self.types[node]),
-                        ", ".join(elts))
-
-    def visit_Set(self, node):
-        if not node.elts:  # empty set
-            return "__builtin__::set()"
-        else:
-            elts = [self.visit(n) for n in node.elts]
-            return "{0}({{ {1} }})".format(
                     Assignable(self.types[node]),
                     ", ".join(elts))
 
+    def visit_Set(self, node):
+        if not node.elts:  # empty set
+            return "pythonic::__builtin__::set()"
+        else:
+            elts = [self.visit(n) for n in node.elts]
+            return "{0}({{ {1} }})".format(
+                Assignable(self.types[node]),
+                ", ".join(elts))
+
     def visit_Dict(self, node):
         if not node.keys:  # empty dict
-            return "__builtin__::dict()"
+            return "pythonic::__builtin__::dict()"
         else:
             keys = [self.visit(n) for n in node.keys]
             values = [self.visit(n) for n in node.values]
             return "{0}({{ {1} }})".format(
-                    Assignable(self.types[node]),
-                    ", ".join("{{ {0}, {1} }}".format(k, v)
-                        for k, v in zip(keys, values)))
+                Assignable(self.types[node]),
+                ", ".join("{{ {0}, {1} }}".format(k, v)
+                          for k, v in zip(keys, values)))
 
     def visit_Tuple(self, node):
         elts = map(self.visit, node.elts or ())
-        return "core::make_tuple({0})".format(", ".join(elts))
+        return "pythonic::types::make_tuple({0})".format(", ".join(elts))
 
     def visit_Compare(self, node):
         left = self.visit(node.left)
@@ -859,24 +846,32 @@ class Cxx(Backend):
     def visit_Call(self, node):
         args = [self.visit(n) for n in node.args]
         func = self.visit(node.func)
-        return "{0}({1})".format(func, ", ".join(args))
+        # special hook for getattr, as we cannot represent it in C++
+        if func == 'pythonic::__builtin__::proxy::getattr{}':
+            return ('pythonic::__builtin__::getattr<{}>({})'
+                    .format('pythonic::types::attr::' + node.args[1].s,
+                            args[0]))
+        else:
+            return "{}({})".format(func, ", ".join(args))
 
     def visit_Num(self, node):
         if type(node.n) == complex:
             return "{0}({1}, {2})".format(
-                    pytype_to_ctype_table[complex],
-                    repr(node.n.real),
-                    repr(node.n.imag))
+                pytype_to_ctype_table[complex],
+                repr(node.n.real),
+                repr(node.n.imag))
         elif type(node.n) == long:
             return 'pythran_long({0})'.format(node.n)
         elif isnan(node.n):
-            return 'pythonic::nan'
+            return 'pythonic::numpy::nan'
+        elif isinf(node.n):
+            return ('+' if node.n > 0 else '-') + 'pythonic::numpy::inf'
         else:
             return repr(node.n) + type_to_suffix.get(type(node.n), "")
 
     def visit_Str(self, node):
         quoted = node.s.replace('"', '\\"').replace('\n', '\\n"\n"')
-        return 'core::string("{0}")'.format(quoted)
+        return 'pythonic::types::str("{0}")'.format(quoted)
 
     def visit_Attribute(self, node):
         def rec(w, n):
@@ -884,27 +879,29 @@ class Cxx(Backend):
                 return w[n.id], (n.id,)
             elif isinstance(n, ast.Attribute):
                 r = rec(w, n.value)
+                if len(r[1]) > 1:
+                    plast, last = r[1][-2:]
+                    if plast == '__builtin__' and last.startswith('__'):
+                        return r[0][n.attr], r[1][:-2] + r[1][-1:] + (n.attr,)
                 return r[0][n.attr], r[1] + (n.attr,)
         obj, path = rec(modules, node)
+        path = ('pythonic',) + path
         return ('::'.join(path) if obj.isliteral()
                 else ('::'.join(path[:-1]) + '::proxy::' + path[-1] + '{}'))
 
     def visit_Subscript(self, node):
         value = self.visit(node.value)
-        # attribute case
-        if metadata.get(node, metadata.Attribute):
-            return "getattr<{0}>({1})".format(node.slice.value.n, value)
         # positive static index case
-        elif (isinstance(node.slice, ast.Index)
+        if (isinstance(node.slice, ast.Index)
                 and isinstance(node.slice.value, ast.Num)
                 and (node.slice.value.n >= 0)
                 and any(isinstance(node.slice.value.n, t)
-                    for t in (int, long))):
+                        for t in (int, long))):
             return "std::get<{0}>({1})".format(node.slice.value.n, value)
         # slice optimization case
         elif (isinstance(node.slice, ast.Slice)
                 and (isinstance(node.ctx, ast.Store)
-                    or node not in self.bounded_expressions)):
+                     or node not in self.bounded_expressions)):
             slice = self.visit(node.slice)
             return "{1}({0})".format(slice, value)
         # extended slice case
@@ -932,9 +929,15 @@ class Cxx(Backend):
         args = []
         for field in ('lower', 'upper', 'step'):
             nfield = getattr(node, field)
-            arg = self.visit(nfield) if nfield else '__builtin__::None'
+            arg = (self.visit(nfield) if nfield
+                   else 'pythonic::__builtin__::None')
             args.append(arg)
-        return "core::slice({},{},{})".format(*args)
+        if node.step is None or (type(node.step) is ast.Num
+                                 and node.step.n == 1):
+            return "pythonic::types::contiguous_slice({},{})".format(args[0],
+                                                                     args[1])
+        else:
+            return "pythonic::types::slice({},{},{})".format(*args)
 
     def visit_Index(self, node):
         return self.visit(node.value)
