@@ -208,7 +208,8 @@ class ExceptHandler(Generable):
         if self.name is None:
             yield "catch(...)"
         else:
-            yield "catch (core::%s const& %s)" % (self.name, self.alias or '')
+            yield "catch (pythonic::types::%s const& %s)" % (self.name,
+                                                             self.alias or '')
         for line in self.body.generate():
             yield line
 
@@ -298,9 +299,8 @@ class AutoFor(Loop):
 
     def intro_line(self):
         return ("for (typename decltype({1})::iterator::reference "
-                "{0}: {1})".format(
-                self.target,
-                self.iter))
+                "{0}: {1})".format(self.target,
+                                   self.iter))
 
 
 # simple statements -----------------------------------------------------------
@@ -339,8 +339,9 @@ class AnnotatedStatement(Generable):
         self.annotations = annotations
 
     def generate(self):
-        for a in self.annotations:
-            yield "#pragma %s" % (a)
+        for directive in self.annotations:
+            pragma = "#pragma " + directive.s
+            yield pragma.format(*directive.deps)
         for s in self.stmt.generate():
             yield s
 
@@ -476,9 +477,8 @@ class BoostPythonModule(object):
 
         self.mod_body.append(func)
         self.init_body.append(
-                Statement(
-                    "boost::python::def(\"%s\", &%s)" % (
-                        name, func.fdecl.name)))
+            Statement("boost::python::def(\"%s\", &%s)" % (name,
+                                                           func.fdecl.name)))
 
     def generate(self):
         """Generate (i.e. yield) the source code of the
