@@ -82,12 +82,10 @@ class PType(Type):
     count = 0
 
     def __init__(self, fun, type):
-        super(PType, self).__init__(
-                fun=fun,
-                type=type,
-                qualifiers=type.qualifiers,
-                name=PType.prefix.format(PType.count)
-                )
+        super(PType, self).__init__(fun=fun,
+                                    type=type,
+                                    qualifiers=type.qualifiers,
+                                    name=PType.prefix.format(PType.count))
         PType.count += 1
 
     def generate(self, ctx):
@@ -95,10 +93,10 @@ class PType(Type):
 
     def instanciate(self, caller, arguments):
         return InstanciatedType(self.fun,
-                self.name,
-                arguments,
-                caller,
-                self.qualifiers)
+                                self.name,
+                                arguments,
+                                caller,
+                                self.qualifiers)
 
 
 class InstanciatedType(Type):
@@ -106,12 +104,10 @@ class InstanciatedType(Type):
     A type instanciated from a parametric type
     """
     def __init__(self, fun, name, arguments, caller, qualifiers):
-        super(InstanciatedType, self).__init__(
-            fun=fun,
-            name=name,
-            arguments=arguments,
-            qualifiers=qualifiers
-            )
+        super(InstanciatedType, self).__init__(fun=fun,
+                                               name=name,
+                                               arguments=arguments,
+                                               qualifiers=qualifiers)
         if fun == caller:
             self.qualifiers.add(Weak)
 
@@ -122,11 +118,9 @@ class InstanciatedType(Type):
         else:
             template_params = ""
 
-        return "typename {0}::type{1}::{2}".format(
-            self.fun.name,
-            template_params,
-            self.name
-            )
+        return "typename {0}::type{1}::{2}".format(self.fun.name,
+                                                   template_params,
+                                                   self.name)
 
 
 class CombinedTypes(Type):
@@ -191,9 +185,8 @@ class ArgumentType(Type):
 typename std::remove_reference<argument_type4>::type>::type
     """
     def __init__(self, num, qualifiers=set()):
-        super(ArgumentType, self).__init__(
-                num=num,
-                qualifiers=qualifiers)
+        super(ArgumentType, self).__init__(num=num,
+                                           qualifiers=qualifiers)
 
     def generate(self, ctx):
         argtype = "argument_type{0}".format(self.num)
@@ -206,10 +199,8 @@ class DependentType(Type):
     A class to be sub-classed by any type that depends on another type
     """
     def __init__(self, of):
-        super(DependentType, self).__init__(
-                of=of,
-                qualifiers=of.qualifiers
-                )
+        super(DependentType, self).__init__(of=of,
+                                            qualifiers=of.qualifiers)
 
 
 class Assignable(DependentType):
@@ -221,11 +212,12 @@ class Assignable(DependentType):
     * assignable types (typically type of a variable)
 
     >>> Assignable(NamedType("long"))
-    typename assignable<long>::type
+    typename pythonic::assignable<long>::type
     """
 
     def generate(self, ctx):
-        return 'typename assignable<{0}>::type'.format(self.of.generate(ctx))
+        return 'typename pythonic::assignable<{0}>::type'.format(
+            self.of.generate(ctx))
 
 
 class Lazy(DependentType):
@@ -235,11 +227,12 @@ class Lazy(DependentType):
     It is used to make a lazy evaluation of numpy expressions
 
     >>> Lazy(NamedType("long"))
-    typename lazy<long>::type
+    typename pythonic::lazy<long>::type
     """
 
     def generate(self, ctx):
-        return 'typename lazy<{0}>::type'.format(self.of.generate(ctx))
+        return 'typename pythonic::lazy<{0}>::type'.format(
+            self.of.generate(ctx))
 
 
 class DeclType(NamedType):
@@ -262,7 +255,7 @@ class ContentType(DependentType):
     Type of the object in a container
 
     >>> ContentType(DeclType('l'))
-    typename content_of<typename std::remove_cv<\
+    typename pythonic::types::content_of<typename std::remove_cv<\
 typename std::remove_reference<decltype(l)>::type>::type>::type
     '''
 
@@ -270,17 +263,17 @@ typename std::remove_reference<decltype(l)>::type>::type>::type
         # the content of a container can be inferred directly
         if type(self.of) in (ListType, SetType, ContainerType):
             return self.of.of.generate(ctx)
-        return 'typename content_of<{0}>::type'.format(
-                ctx(self.of).generate(ctx))
+        return 'typename pythonic::types::content_of<{0}>::type'.format(
+            ctx(self.of).generate(ctx))
 
 
 class IteratorContentType(DependentType):
     '''
     Type of an iterator over the content of a container
 
-    >>> IteratorContentType(NamedType('string'))
+    >>> IteratorContentType(NamedType('str'))
     typename std::remove_cv<typename std::iterator_traits<\
-typename std::remove_reference<string>::type::iterator>::value_type>::type
+typename std::remove_reference<str>::type::iterator>::value_type>::type
     '''
 
     def generate(self, ctx):
@@ -290,11 +283,31 @@ typename std::remove_reference<string>::type::iterator>::value_type>::type
                 return ctx(NamedType('long')).generate(ctx)
         iterator_value_type = ctx(self.of).generate(ctx)
         return 'typename std::remove_cv<{0}>::type'.format(
-                'typename std::iterator_traits<{0}>::value_type'.format(
-                    'typename std::remove_reference<{0}>::type::iterator'.
-                    format(iterator_value_type)
-                    )
+            'typename std::iterator_traits<{0}>::value_type'.format(
+                'typename std::remove_reference<{0}>::type::iterator'.format(
+                    iterator_value_type)
                 )
+            )
+
+
+class GetAttr(Type):
+    '''
+    Type of a named attribute
+
+    >>> GetAttr(NamedType('complex'), 'real')
+    decltype(pythonic::__builtin__::getattr<pythonic::types::attr::real>\
+(std::declval<complex>()))
+    '''
+    def __init__(self, param, attr):
+        super(GetAttr, self).__init__(
+            qualifiers=param.qualifiers,
+            param=param,
+            attr=attr)
+
+    def generate(self, ctx):
+        return ('decltype(pythonic::__builtin__::getattr<{}>({}))'
+                .format('pythonic::types::attr::' + self.attr,
+                        'std::declval<' + self.param.generate(ctx) + '>()'))
 
 
 class ReturnType(Type):
@@ -307,10 +320,9 @@ class ReturnType(Type):
     def __init__(self, ftype, args):
         args_qualifiers = [arg.qualifiers for arg in args]
         super(ReturnType, self).__init__(
-                qualifiers=ftype.qualifiers.union(*args_qualifiers),
-                ftype=ftype,
-                args=args
-                )
+            qualifiers=ftype.qualifiers.union(*args_qualifiers),
+            ftype=ftype,
+            args=args)
 
     def generate(self, ctx):
         # the return type of a constructor is obvious
@@ -328,38 +340,22 @@ class ElementType(Type):
     >>> t = TupleType([NamedType('int'), NamedType('str')])
     >>> ElementType(1, t)
     typename std::tuple_element<1,typename std::remove_reference<\
-decltype(core::make_tuple(std::declval<int>(), std::declval<str>()))>\
-::type>::type
+decltype(pythonic::types::make_tuple(std::declval<int>(), \
+std::declval<str>()))>::type>::type
     '''
 
     def __init__(self, index, of):
-        super(ElementType, self).__init__(
-                qualifiers=of.qualifiers,
-                of=of,
-                index=index
-                )
+        super(ElementType, self).__init__(qualifiers=of.qualifiers,
+                                          of=of,
+                                          index=index)
 
     def generate(self, ctx):
         return 'typename std::tuple_element<{0},{1}>::type'.format(
-                self.index,
-                'typename std::remove_reference<{0}>::type'.format(
-                    ctx(self.of).generate(ctx)
-                    )
+            self.index,
+            'typename std::remove_reference<{0}>::type'.format(
+                ctx(self.of).generate(ctx)
                 )
-
-
-class AttributeType(ElementType):
-    '''
-    Type of the ith attribute of an object
-
-    >>> AttributeType(0, NamedType('complex'))
-    typename attribute_element<0,complex>::type
-    '''
-
-    def generate(self, ctx):
-        return 'typename attribute_element<{0},{1}>::type'.format(
-                self.index,
-                ctx(self.of).generate(ctx))
+            )
 
 
 class ListType(DependentType):
@@ -367,11 +363,11 @@ class ListType(DependentType):
     Type holding a list of stuff of the same type
 
     >>> ListType(NamedType('int'))
-    core::list<int>
+    pythonic::types::list<int>
     '''
 
     def generate(self, ctx):
-        return 'core::list<{0}>'.format(ctx(self.of).generate(ctx))
+        return 'pythonic::types::list<{0}>'.format(ctx(self.of).generate(ctx))
 
 
 class SetType(DependentType):
@@ -379,11 +375,11 @@ class SetType(DependentType):
     Type holding a set of stuff of the same type
 
     >>> SetType(NamedType('int'))
-    core::set<int>
+    pythonic::types::set<int>
     '''
 
     def generate(self, ctx):
-        return 'core::set<{0}>'.format(ctx(self.of).generate(ctx))
+        return 'pythonic::types::set<{0}>'.format(ctx(self.of).generate(ctx))
 
 
 class TupleType(Type):
@@ -391,7 +387,8 @@ class TupleType(Type):
     Type holding a tuple of stuffs of various types
 
     >>> TupleType([NamedType('int'), NamedType('bool')])
-    decltype(core::make_tuple(std::declval<int>(), std::declval<bool>()))
+    decltype(pythonic::types::make_tuple(std::declval<int>(), \
+std::declval<bool>()))
     '''
     def __init__(self, ofs):
         if ofs:
@@ -404,7 +401,8 @@ class TupleType(Type):
     def generate(self, ctx):
         elts = (ctx(of).generate(ctx) for of in self.ofs)
         telts = ('std::declval<{0}>()'.format(elt) for elt in elts)
-        return 'decltype(core::make_tuple({0}))'.format(", ".join(telts))
+        return 'decltype(pythonic::types::make_tuple({0}))'.format(
+            ", ".join(telts))
 
 
 class DictType(Type):
@@ -412,20 +410,20 @@ class DictType(Type):
     Type holding a dict of stuff of the same key and value type
 
     >>> DictType(NamedType('int'), NamedType('float'))
-    core::dict<int,float>
+    pythonic::types::dict<int,float>
     '''
 
     def __init__(self, of_key, of_value):
         super(DictType, self).__init__(
-                qualifiers=of_key.qualifiers.union(of_value.qualifiers),
-                of_key=of_key,
-                of_value=of_value
-                )
+            qualifiers=of_key.qualifiers.union(of_value.qualifiers),
+            of_key=of_key,
+            of_value=of_value
+            )
 
     def generate(self, ctx):
-        return 'core::dict<{0},{1}>'.format(
-                ctx(self.of_key).generate(ctx),
-                ctx(self.of_value).generate(ctx))
+        return 'pythonic::types::dict<{0},{1}>'.format(
+            ctx(self.of_key).generate(ctx),
+            ctx(self.of_value).generate(ctx))
 
 
 class ContainerType(DependentType):
@@ -438,7 +436,7 @@ class ContainerType(DependentType):
 
     def generate(self, ctx):
         return 'container<typename std::remove_reference<{0}>::type>'.format(
-                ctx(self.of).generate(ctx))
+            ctx(self.of).generate(ctx))
 
 
 class IndexableType(DependentType):
@@ -463,11 +461,11 @@ class ExpressionType(Type):
     '''
     def __init__(self, op, exprs):
         super(ExpressionType, self).__init__(
-                qualifiers=set.union(*[expr.qualifiers for expr in exprs]),
-                op=op,
-                exprs=exprs)
+            qualifiers=set.union(*[expr.qualifiers for expr in exprs]),
+            op=op,
+            exprs=exprs)
 
     def generate(self, ctx):
         texprs = (ctx(expr).generate(ctx) for expr in self.exprs)
         return 'decltype({0})'.format(
-                self.op(*["std::declval<{0}>()".format(t) for t in texprs]))
+            self.op(*["std::declval<{0}>()".format(t) for t in texprs]))
