@@ -6,8 +6,8 @@
 //                 See accompanying file LICENSE.txt or copy at
 //                     http://www.boost.org/LICENSE_1_0.txt
 //==============================================================================
-#ifndef NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_1_HPP_INCLUDED
-#define NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_1_HPP_INCLUDED
+#ifndef NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_2_HPP_INCLUDED
+#define NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_2_HPP_INCLUDED
 
 #include <boost/simd/sdk/constant/constant.hpp>
 #include <boost/simd/sdk/constant/register.hpp>
@@ -16,14 +16,14 @@ namespace nt2
 {
   namespace tag
   {
-    // 1.57079632673412561417e+00
-    BOOST_SIMD_CONSTANT_REGISTER( Pio2_1, double
-                                , 1, 0x3fc90f80
-                                , 0x3FF921FB54400000ll
+    //6.07710050630396597660e-11
+    BOOST_SIMD_CONSTANT_REGISTER( Pio2_2, double
+                                , 0, 0x37354400
+                                , 0x3DD0B4611A600000ll
                                 );
   }
 
-  BOOST_SIMD_CONSTANT_IMPLEMENTATION(tag::Pio2_1, Pio2_1);
+  BOOST_SIMD_CONSTANT_IMPLEMENTATION(tag::Pio2_2, Pio2_2);
 }
 
 #endif
