@@ -6,8 +6,8 @@
 //                 See accompanying file LICENSE.txt or copy at
 //                     http://www.boost.org/LICENSE_1_0.txt
 //==============================================================================
-#ifndef NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_2T_HPP_INCLUDED
-#define NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_2T_HPP_INCLUDED
+#ifndef NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_3T_HPP_INCLUDED
+#define NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIO2_3T_HPP_INCLUDED
 
 #include <boost/simd/sdk/constant/constant.hpp>
 #include <boost/simd/sdk/constant/register.hpp>
@@ -16,14 +16,14 @@ namespace nt2
 {
   namespace tag
   {
-    // 2.02226624879595063154e-21
-    BOOST_SIMD_CONSTANT_REGISTER( Pio2_2t, double
-                                , 0, 0x2e85a308
-                                , 0x3BA3198A2E037073ll
+    // 8.47842766036889956997e-32
+    BOOST_SIMD_CONSTANT_REGISTER( Pio2_3t, double
+                                , 0, 0x248d3132
+                                , 0x397B839A252049C1ll
                                 );
   }
 
-  BOOST_SIMD_CONSTANT_IMPLEMENTATION(tag::Pio2_2t, Pio2_2t);
+  BOOST_SIMD_CONSTANT_IMPLEMENTATION(tag::Pio2_3t, Pio2_3t);
 }
 
 #endif
