@@ -6,8 +6,8 @@
 //                 See accompanying file LICENSE.txt or copy at
 //                     http://www.boost.org/LICENSE_1_0.txt
 //==============================================================================
-#ifndef NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIX2_3_HPP_INCLUDED
-#define NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIX2_3_HPP_INCLUDED
+#ifndef NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIX2_2_HPP_INCLUDED
+#define NT2_TOOLBOX_TRIGONOMETRIC_CONSTANTS_PIX2_2_HPP_INCLUDED
 
 #include <boost/simd/sdk/constant/constant.hpp>
 #include <boost/simd/sdk/constant/register.hpp>
@@ -17,13 +17,13 @@ namespace nt2
   namespace tag
   {
     // 12.56637061
-    BOOST_SIMD_CONSTANT_REGISTER( Pix2_3, double
-                                , 1, 0x31885a30
-                                , 0x3bc3198a2e037073ll//3b498a2e03707345ll
+    BOOST_SIMD_CONSTANT_REGISTER( Pix2_2, double
+                                , 1, 0x38daa000
+                                , 0x3df0b4611a600000LL
                                 );
   }
 
-  BOOST_SIMD_CONSTANT_IMPLEMENTATION(tag::Pix2_3, Pix2_3);
+  BOOST_SIMD_CONSTANT_IMPLEMENTATION(tag::Pix2_2, Pix2_2);
 }
 
 #endif
