@@ -6,6 +6,10 @@ import ply.lex as lex
 import ply.yacc as yacc
 import os.path
 from numpy import array
+from numpy import uint8, uint16, uint32, uint64
+from numpy import int8, int16, int32, int64
+from numpy import float32, float64
+from numpy import complex64, complex128
 
 
 class SpecParser:
@@ -19,21 +23,33 @@ class SpecParser:
 
     ## lex part
     reserved = {
-            'pythran': 'PYTHRAN',
-            'export': 'EXPORT',
-            'list': 'LIST',
-            'set': 'SET',
-            'dict': 'DICT',
-            'str': 'STR',
-            'bool': 'BOOL',
-            'complex': 'COMPLEX',
-            'int': 'INT',
-            'long': 'LONG',
-            'float': 'FLOAT',
-            }
+        'pythran': 'PYTHRAN',
+        'export': 'EXPORT',
+        'list': 'LIST',
+        'set': 'SET',
+        'dict': 'DICT',
+        'str': 'STR',
+        'bool': 'BOOL',
+        'complex': 'COMPLEX',
+        'int': 'INT',
+        'long': 'LONG',
+        'float': 'FLOAT',
+        'uint8': 'UINT8',
+        'uint16': 'UINT16',
+        'uint32': 'UINT32',
+        'uint64': 'UINT64',
+        'int8': 'INT8',
+        'int16': 'INT16',
+        'int32': 'INT32',
+        'int64': 'INT64',
+        'float32': 'FLOAT32',
+        'float64': 'FLOAT64',
+        'complex64': 'COMPLEX64',
+        'complex128': 'COMPLEX128',
+        }
     tokens = (['IDENTIFIER', 'SHARP', 'COMMA', 'COLUMN', 'LPAREN', 'RPAREN']
-            + list(reserved.values())
-            + ['LARRAY', 'RARRAY'])
+              + list(reserved.values())
+              + ['LARRAY', 'RARRAY'])
 
     # token <> regexp binding
     t_SHARP = r'\#'
@@ -110,7 +126,7 @@ class SpecParser:
             p[0] = {p[1]: p[3]}
         else:
             raise SyntaxError("Invalid Pythran spec. "
-                    "Unknown text '{0}'".format(p.value))
+                              "Unknown text '{0}'".format(p.value))
 
     def p_term(self, p):
         '''term : STR
@@ -118,7 +134,19 @@ class SpecParser:
                 | COMPLEX
                 | INT
                 | LONG
-                | FLOAT'''
+                | FLOAT
+                | UINT8
+                | UINT16
+                | UINT32
+                | UINT64
+                | INT8
+                | INT16
+                | INT32
+                | INT64
+                | FLOAT32
+                | FLOAT64
+                | COMPLEX64
+                | COMPLEX128'''
         p[0] = eval(p[1])
 
     def p_error(self, p):
@@ -131,10 +159,9 @@ class SpecParser:
 
     def __init__(self, **kwargs):
         self.lexer = lex.lex(module=self, debug=0)
-        self.parser = yacc.yacc(
-                module=self,
-                debug=0,
-                tabmodule='pythran.parsetab')
+        self.parser = yacc.yacc(module=self,
+                                debug=0,
+                                tabmodule='pythran.parsetab')
 
     def __call__(self, path):
         self.exports = dict()
@@ -150,11 +177,9 @@ class SpecParser:
                                   for line in data.split('\n')))
         self.parser.parse(pythran_data, lexer=self.lexer)
         if not self.exports:
-            err = SyntaxError(
-                    "Pythran spec error: no pythran specification")
-            if self.input_file:
-                err.filename = self.input_file
-            raise err
+            import logging
+            logging.warn("No pythran specification, "
+                         "no function will be exported")
         return self.exports
 
 
