@@ -14,13 +14,11 @@ from cxxgen import *
 import ast
 from middlend import refine
 from backend import Cxx
-from syntax import check_syntax
-from passes import NormalizeIdentifiers
-from openmp import GatherOMPData
+import frontend
 from config import cfg
 from passmanager import PassManager
 from numpy import get_include
-from typing import extract_constructed_types, pytype_to_ctype
+from typing import extract_constructed_types, pytype_to_ctype, pytype_to_deps
 from tables import pythran_ward, functions
 from intrinsic import ConstExceptionIntr
 
@@ -41,6 +39,15 @@ def _extract_all_constructed_types(v):
                   key=len)
 
 
+def _extract_specs_dependencies(specs):
+    deps = set()
+    for _, signatures in specs.iteritems():
+        for _, signature in enumerate(signatures):
+            for t in signature:
+                deps.update(pytype_to_deps(t))
+    return deps
+
+
 def _parse_optimization(optimization):
     '''Turns an optimization of the form
         my_optim
@@ -63,7 +70,7 @@ def _numpy_cppflags():
 def _pythran_cppflags():
     curr_dir = os.path.dirname(os.path.dirname(__file__))
     get = lambda *x: '-I' + os.path.join(curr_dir, *x)
-    return [get('.'), get('pythran'), get('pythran', 'pythonic++')]
+    return [get('.'), get('pythran')]
 
 
 def _python_ldflags():
@@ -136,18 +143,9 @@ def generate_cxx(module_name, code, specs=None, optimizations=None):
 
     '''
     pm = PassManager(module_name)
-    # hacky way to turn OpenMP comments into strings
-    code = re.sub(r'(\s*)#(omp\s[^\n]+)', r'\1"\2"', code)
 
-    # font end
-    ir = ast.parse(code)
-
-    # parse openmp directive
-    pm.apply(GatherOMPData, ir)
-
-    # avoid conflicts with cxx keywords
-    renamings = pm.apply(NormalizeIdentifiers, ir)
-    check_syntax(ir)
+    # front end
+    ir, renamings = frontend.parse(pm, code)
 
     # middle-end
     optimizations = (optimizations or
@@ -159,7 +157,19 @@ def generate_cxx(module_name, code, specs=None, optimizations=None):
     content = pm.dump(Cxx, ir)
 
     # instanciate the meta program
-    if specs:
+    if specs is None:
+
+        class Generable:
+            def __init__(self, content):
+                self.content = content
+
+            def __str__(self):
+                return str(self.content)
+
+            generate = __str__
+
+        mod = Generable(content)
+    else:
         # uniform typing
         for fname, signatures in specs.items():
             if not isinstance(signatures, tuple):
@@ -168,15 +178,17 @@ def generate_cxx(module_name, code, specs=None, optimizations=None):
         mod = BoostPythonModule(module_name)
         mod.use_private_namespace = False
         # very low value for max_arity leads to various bugs
-        max_arity = max(4, max(max(map(len, s)) for s in specs.itervalues()))
+        min_val = 2
+        specs_max = [max(map(len, s)) for s in specs.itervalues()]
+        max_arity = max([min_val] + specs_max)
         mod.add_to_preamble([Define("BOOST_PYTHON_MAX_ARITY", max_arity)])
         mod.add_to_preamble([Define("BOOST_SIMD_NO_STRICT_ALIASING", "1")])
+        mod.add_to_preamble([Include("pythonic/core.hpp")])
+        mod.add_to_preamble([Include("pythonic/python/core.hpp")])
+        mod.add_to_preamble(map(Include, _extract_specs_dependencies(specs)))
         mod.add_to_preamble(content.body)
         mod.add_to_init([
-            Statement('import_array()'),
-            Statement('boost::python::implicitly_convertible<std::string,'
-                      + 'pythonic::core::string>()')]
-        )
+            Line('#ifdef PYTHONIC_TYPES_NDARRAY_HPP\nimport_array()\n#endif')])
 
         # topologically sorted exceptions based on the inheritance hierarchy.
         # needed because otherwise boost python register_exception handlers
@@ -197,10 +209,14 @@ def generate_cxx(module_name, code, specs=None, optimizations=None):
 
         sorted_exceptions = nx.topological_sort(exceptions)
         mod.add_to_init([
-            Statement(
-                'boost::python::register_exception_translator<' +
-                'pythonic::core::%s>(&translate_%s)' %
-                (n.__name__, n.__name__)) for n in sorted_exceptions])
+            # register exception only if they can be raise from C++ world to
+            # Python world. Preprocessors variables are set only if deps
+            # analysis detect that this exception can be raised
+            Line('#ifdef PYTHONIC_BUILTIN_%s_HPP\n'
+                 'boost::python::register_exception_translator<'
+                 'pythonic::types::%s>(&pythonic::translate_%s);\n'
+                 '#endif' % (n.__name__.upper(), n.__name__, n.__name__)
+                 ) for n in sorted_exceptions])
 
         for function_name, signatures in specs.iteritems():
             internal_func_name = renamings.get(function_name,
@@ -218,14 +234,15 @@ def generate_cxx(module_name, code, specs=None, optimizations=None):
                                                     internal_func_name,
                                                     "<{0}>".format(args_list)
                                                     if has_arguments else "")
-                result_type = ("typename std::remove_reference"
-                               + "<typename {0}::result_type>::type".format(
-                                 specialized_fname))
+                result_type = ("typename std::remove_cv<"
+                               "typename std::remove_reference"
+                               "<typename {0}::result_type>::type"
+                               ">::type").format(specialized_fname)
                 mod.add_to_init(
-                    [Statement("python_to_pythran<{0}>()".format(t))
+                    [Statement("pythonic::python_to_pythran<{0}>()".format(t))
                      for t in _extract_all_constructed_types(signature)])
                 mod.add_to_init([Statement(
-                    "pythran_to_python<{0}>()".format(result_type))])
+                    "pythonic::pythran_to_python<{0}>()".format(result_type))])
                 mod.add_function(
                     FunctionBody(
                         FunctionDeclaration(
@@ -241,8 +258,9 @@ def generate_cxx(module_name, code, specs=None, optimizations=None):
                     ),
                     function_name
                 )
-    else:
-        mod = content
+        # call __init__() to execute top-level statements
+        init_call = '::'.join([pythran_ward + module_name, '__init__()()'])
+        mod.add_to_init([Statement(init_call)])
     return mod
 
 
@@ -259,15 +277,6 @@ def compile_cxxfile(cxxfile, module_so=None, **kwargs):
     _cxxflags = cxxflags() + kwargs.get('cxxflags', [])
     _ldflags = ldflags() + kwargs.get('ldflags', [])
 
-    # workaround g++-4.8 bug
-    try:
-        extra_flags = ['-ftrack-macro-expansion=0']
-        check_call([compiler, cxxfile, '-E'] + extra_flags,
-                     stdout=file(devnull), stderr=file(devnull))
-        _cppflags.extend(extra_flags)
-    except CalledProcessError:
-        pass
-
     # Get output filename from input filename if not set
     module_so = module_so or (os.path.splitext(cxxfile)[0] + ".so")
     try:
@@ -315,7 +324,8 @@ def compile_pythrancode(module_name, pythrancode, specs=None,
 
     # Autodetect the Pythran spec if not given as parameter
     from spec import spec_parser
-    specs = spec_parser(pythrancode) if specs is None else specs
+    if specs is None:
+        specs = spec_parser(pythrancode)
 
     # Generate C++, get a BoostPythonModule object
     module = generate_cxx(module_name, pythrancode, specs, opts)
@@ -366,6 +376,6 @@ def test_compile():
     '''
     module_so = compile_cxxcode("\n".join([
         "#define BOOST_PYTHON_MAX_ARITY 4",
-        "#include <pythran/pythran.h>"
+        "#include <pythonic/core.hpp>"
         ]))
     module_so and os.remove(module_so)
