@@ -8,12 +8,12 @@ import ast
 from numpy import ndarray
 import networkx as nx
 
-from tables import pytype_to_ctype_table, operator_to_lambda
-from tables import modules, methods, functions
-from analysis import GlobalDeclarations, YieldPoints, LocalDeclarations
-from analysis import OrderedGlobalDeclarations, ModuleAnalysis, StrictAliases
-from analysis import LazynessAnalysis
-from passes import Transformation
+from tables import (pytype_to_ctype_table, operator_to_lambda, modules,
+                    methods, functions)
+from analyses import (GlobalDeclarations, YieldPoints, LocalDeclarations,
+                      OrderedGlobalDeclarations, StrictAliases,
+                      LazynessAnalysis)
+from passmanager import ModuleAnalysis, Transformation
 from syntax import PythranSyntaxError
 from cxxtypes import *
 from intrinsic import UserFunction, MethodIntr
@@ -40,27 +40,47 @@ if not "has_path" in nx.__dict__:
 def pytype_to_ctype(t):
     '''python -> c++ type binding'''
     if isinstance(t, list):
-        return 'core::list<{0}>'.format(pytype_to_ctype(t[0]))
+        return 'pythonic::types::list<{0}>'.format(pytype_to_ctype(t[0]))
     if isinstance(t, set):
-        return 'core::set<{0}>'.format(pytype_to_ctype(list(t)[0]))
+        return 'pythonic::types::set<{0}>'.format(pytype_to_ctype(list(t)[0]))
     elif isinstance(t, dict):
         tkey, tvalue = t.items()[0]
-        return 'core::dict<{0},{1}>'.format(pytype_to_ctype(tkey),
-                                            pytype_to_ctype(tvalue))
+        return 'pythonic::types::dict<{0},{1}>'.format(pytype_to_ctype(tkey),
+                                                       pytype_to_ctype(tvalue))
     elif isinstance(t, tuple):
-        return 'decltype(core::make_tuple({0}))'.format(
-                ", ".join('std::declval<{}>()'.format(
-                    pytype_to_ctype(_)) for _ in t)
-                )
+        return 'decltype(pythonic::types::make_tuple({0}))'.format(
+            ", ".join('std::declval<{}>()'.format(
+                pytype_to_ctype(_)) for _ in t)
+            )
     elif isinstance(t, ndarray):
-        return 'core::ndarray<{0},{1}>'.format(pytype_to_ctype(t.flat[0]),
-                                               t.ndim)
+        return 'pythonic::types::ndarray<{0},{1}>'.format(
+            pytype_to_ctype(t.flat[0]), t.ndim)
     elif t in pytype_to_ctype_table:
         return pytype_to_ctype_table[t]
     else:
         raise NotImplementedError("{0}:{1}".format(type(t), t))
 
 
+def pytype_to_deps(t):
+    '''python -> c++ type binding'''
+    if isinstance(t, list):
+        return {'pythonic/types/list.hpp'}.union(pytype_to_deps(t[0]))
+    if isinstance(t, set):
+        return {'pythonic/types/set.hpp'}.union(pytype_to_deps(list(t)[0]))
+    elif isinstance(t, dict):
+        tkey, tvalue = t.items()[0]
+        return {'pythonic/types/dict.hpp'}.union(pytype_to_deps(tkey),
+                                                 pytype_to_deps(tvalue))
+    elif isinstance(t, tuple):
+        return {'pythonic/types/tuple.hpp'}.union(*map(pytype_to_deps, t))
+    elif isinstance(t, ndarray):
+        return {'pythonic/types/ndarray.hpp'}.union(pytype_to_deps(t[0]))
+    elif t in pytype_to_ctype_table:
+        return {'pythonic/types/{}.hpp'.format(t.__name__)}
+    else:
+        raise NotImplementedError("{0}:{1}".format(type(t), t))
+
+
 def extract_constructed_types(t):
     if isinstance(t, list) or isinstance(t, ndarray):
         return [pytype_to_ctype(t)] + extract_constructed_types(t[0])
@@ -116,7 +136,6 @@ class TypeDependencies(ModuleAnalysis):
     def visit_FunctionDef(self, node):
         assert self.current_function is None
         self.current_function = node
-        self.result.add_node(node)
         self.naming = dict()
         self.in_cond = False  # True when we are in a if, while or for
         self.generic_visit(node)
@@ -135,11 +154,11 @@ class TypeDependencies(ModuleAnalysis):
             v = self.visit(node.value)
             for dep_set in v:
                 if dep_set:
-                    [self.result.add_edge(dep,
-                            self.current_function) for dep in dep_set]
+                    for dep in dep_set:
+                        self.result.add_edge(dep, self.current_function)
                 else:
                     self.result.add_edge(TypeDependencies.NoDeps,
-                                            self.current_function)
+                                         self.current_function)
 
     visit_Yield = visit_Return
 
@@ -247,12 +266,12 @@ class Reorder(Transformation):
     '''
     def __init__(self):
         Transformation.__init__(self, TypeDependencies,
-                OrderedGlobalDeclarations)
+                                OrderedGlobalDeclarations)
 
     def prepare(self, node, ctx):
         super(Reorder, self).prepare(node, ctx)
         none_successors = self.type_dependencies.successors(
-                TypeDependencies.NoDeps)
+            TypeDependencies.NoDeps)
         candidates = sorted(none_successors)
         while candidates:
             new_candidates = list()
@@ -268,8 +287,6 @@ class Reorder(Transformation):
                     # nx.write_dot(self.type_dependencies,"b.dot")
                 if not n in self.type_dependencies.successors(n):
                     new_candidates.extend(self.type_dependencies.successors(n))
-                else:
-                    pass
             candidates = new_candidates
 
     def visit_Module(self, node):
@@ -281,15 +298,14 @@ class Reorder(Transformation):
             else:
                 newbody.append(stmt)
             try:
-                newdef = (f for f in nx.topological_sort(
+                newdef = nx.topological_sort(
                     self.type_dependencies,
-                    self.ordered_global_declarations
-                    ))
+                    self.ordered_global_declarations)
                 newdef = [f for f in newdef if isinstance(f, ast.FunctionDef)]
 
             except nx.exception.NetworkXUnfeasible:
                 raise PythranSyntaxError("Infinite function recursion",
-                                        stmt)
+                                         stmt)
         assert set(newdef) == set(olddef)
         node.body = newbody + newdef
         return node
@@ -311,6 +327,7 @@ class Types(ModuleAnalysis):
         self.current_global_declarations = dict()
         self.max_recompute = 1  # max number of use to be lazy
         ModuleAnalysis.__init__(self, StrictAliases, LazynessAnalysis)
+        self.curr_locals_declaration = None
 
     def prepare(self, node, ctx):
         self.passmanager.apply(Reorder, node, ctx)
@@ -323,10 +340,7 @@ class Types(ModuleAnalysis):
 
     def run(self, node, ctx):
         super(Types, self).run(node, ctx)
-        final_types = {k: self.result[k]
-                if k in self.result
-                else v
-                for k, v in self.result.iteritems()}
+        final_types = self.result.copy()
         for head in self.current_global_declarations.itervalues():
             if head not in final_types:
                 final_types[head] = "void"
@@ -365,36 +379,33 @@ class Types(ModuleAnalysis):
                 #    _, signature = functions.get(func.attr, [(None, None)])[0]
                 if signature:
                     return_alias = (signature.return_alias
-                            and signature.return_alias(n))
+                                    and signature.return_alias(n))
                     if return_alias:  # else new location -> unboundable
                         assert len(return_alias), 'Too many return aliases'
                         return self.node_to_id(list(return_alias)[0], depth)
-            raise UnboundableRValue()
-
-        else:
-            raise UnboundableRValue()
+        raise UnboundableRValue()
 
     def isargument(self, node):
         """ checks whether node aliases to a parameter"""
         try:
             node_id, _ = self.node_to_id(node)
             return (node_id in self.name_to_nodes and
-                           any([isinstance(n, ast.Name) and
-                           isinstance(n.ctx, ast.Param)
-                           for n in self.name_to_nodes[node_id]]))
+                    any([isinstance(n, ast.Name) and
+                         isinstance(n.ctx, ast.Param)
+                         for n in self.name_to_nodes[node_id]]))
         except UnboundableRValue:
                 return False
 
     def combine(self, node, othernode, op=None, unary_op=None, register=False):
         if register and node in self.strict_aliases:
             self.combine_(node, othernode, op or operator.add,
-                    unary_op or (lambda x: x), register)
+                          unary_op or (lambda x: x), register)
             for a in self.strict_aliases[node].aliases:
                 self.combine_(a, othernode, op or operator.add,
-                        unary_op or (lambda x: x), register)
+                              unary_op or (lambda x: x), register)
         else:
             self.combine_(node, othernode, op or operator.add,
-                    unary_op or (lambda x: x), register)
+                          unary_op or (lambda x: x), register)
 
     def combine_(self, node, othernode, op, unary_op, register):
         try:
@@ -409,7 +420,7 @@ class Types(ModuleAnalysis):
 
                 # update the type to reflect container nesting
                 unary_op = lambda x: reduce(lambda t, n: ContainerType(t),
-                              xrange(depth), former_unary_op(x))
+                                            xrange(depth), former_unary_op(x))
 
             if isinstance(othernode, ast.FunctionDef):
                 new_type = NamedType(othernode.name)
@@ -435,8 +446,9 @@ class Types(ModuleAnalysis):
                                 translated_othernode = ast.Name(
                                     '__fake__', ast.Load())
                                 s.result[translated_othernode] = (
-                                 parametric_type.instanciate(
-                                 s.current, [s.result[arg] for arg in n.args]))
+                                    parametric_type.instanciate(
+                                        s.current,
+                                        [s.result[arg] for arg in n.args]))
                                 # look for modified argument
                                 for p, effective_arg in enumerate(n.args):
                                     formal_arg = args[p]
@@ -445,8 +457,8 @@ class Types(ModuleAnalysis):
                                         break
                                 try:
                                     s.combine(translated_node,
-                                        translated_othernode,
-                                        op, unary_op, register=True)
+                                              translated_othernode,
+                                              op, unary_op, register=True)
                                 except NotImplementedError:
                                     pass
                                     # this may fail when the effective
@@ -473,6 +485,9 @@ class Types(ModuleAnalysis):
             raise
 
     def visit_FunctionDef(self, node):
+        self.curr_locals_declaration = self.passmanager.gather(
+            LocalDeclarations,
+            node)
         self.current = node
         self.typedefs = list()
         self.name_to_nodes = {arg.id: {arg} for arg in node.args.args}
@@ -500,11 +515,11 @@ class Types(ModuleAnalysis):
         return_type = self.result.get(node, NamedType("void"))
         self.result[node] = (Assignable(return_type), self.typedefs)
         for k in self.passmanager.gather(LocalDeclarations, node):
-            lazy_res = self.lazyness_analysis[k.id]
-            if lazy_res <= self.max_recompute:
-                self.result[k] = Lazy(self.result[k])
-            else:
-                self.result[k] = Assignable(self.result[k])
+            self.result[k] = self.get_qualifier(k)(self.result[k])
+
+    def get_qualifier(self, node):
+        lazy_res = self.lazyness_analysis[node.id]
+        return Lazy if lazy_res <= self.max_recompute else Assignable
 
     def visit_Return(self, node):
         self.generic_visit(node)
@@ -522,6 +537,8 @@ class Types(ModuleAnalysis):
         self.visit(node.value)
         for t in node.targets:
             self.combine(t, node.value, register=True)
+            if t in self.curr_locals_declaration:
+                self.result[t] = self.get_qualifier(t)(self.result[t])
             if isinstance(t, ast.Subscript):
                 if self.visit_AssignedSubscript(t):
                     for alias in self.strict_aliases[t.value].aliases:
@@ -531,23 +548,24 @@ class Types(ModuleAnalysis):
     def visit_AugAssign(self, node):
         self.visit(node.value)
         self.combine(node.target, node.value,
-            lambda x, y: x + ExpressionType(operator_to_lambda[type(node.op)],
-                [x, y]), register=True)
+                     lambda x, y: x + ExpressionType(
+                         operator_to_lambda[type(node.op)],
+                         [x, y]), register=True)
         if isinstance(node.target, ast.Subscript):
             if self.visit_AssignedSubscript(node.target):
                 for alias in self.strict_aliases[node.target.value].aliases:
                     fake = ast.Subscript(alias, node.target.value, ast.Store())
                     self.combine(fake,
-                            node.value,
-                            lambda x, y: x + ExpressionType(
-                                operator_to_lambda[type(node.op)],
-                                [x, y]),
-                            register=True)
+                                 node.value,
+                                 lambda x, y: x + ExpressionType(
+                                     operator_to_lambda[type(node.op)],
+                                     [x, y]),
+                                 register=True)
 
     def visit_For(self, node):
         self.visit(node.iter)
         self.combine(node.target, node.iter,
-            unary_op=IteratorContentType, register=True)
+                     unary_op=IteratorContentType, register=True)
         node.body and map(self.visit, node.body)
         node.orelse and map(self.visit, node.orelse)
 
@@ -559,7 +577,7 @@ class Types(ModuleAnalysis):
         self.generic_visit(node)
         wl, wr = [self.result[x].isweak() for x in (node.left, node.right)]
         if (isinstance(node.op, ast.Add) and any([wl, wr])
-            and not all([wl, wr])):
+                and not all([wl, wr])):
         # assumes the + operator always has the same operand type
         # on left and right side
             F = operator.add
@@ -587,10 +605,10 @@ class Types(ModuleAnalysis):
         all_compare = zip(node.ops, node.comparators)
         for op, comp in all_compare:
             self.combine(node, comp,
-                    unary_op=lambda x: ExpressionType(
-                        operator_to_lambda[type(op)],
-                        [self.result[node.left], x])
-                    )
+                         unary_op=lambda x: ExpressionType(
+                             operator_to_lambda[type(op)],
+                             [self.result[node.left], x])
+                         )
 
     def visit_Call(self, node):
         self.generic_visit(node)
@@ -604,12 +622,13 @@ class Types(ModuleAnalysis):
                 bounded_function = list(self.strict_aliases[a0].aliases)[0]
                 fake_name = ast.Name(bounded_name, ast.Load())
                 fake_node = ast.Call(fake_name, alias.args[1:] + node.args,
-                    [], None, None)
+                                     [], None, None)
                 self.combiners[bounded_function].combiner(self, fake_node)
                 # force recombination of binded call
                 for n in self.name_to_nodes[node.func.id]:
                     self.result[n] = ReturnType(self.result[alias.func],
-                        [self.result[arg] for arg in alias.args])
+                                                [self.result[arg]
+                                                 for arg in alias.args])
             # handle backward type dependencies from function calls
             else:
                 self.combiners[alias].combiner(self, node)
@@ -625,8 +644,13 @@ class Types(ModuleAnalysis):
                     self.result[node] = self.result[alias]
                     return
 
+        # special handler for getattr: use the attr name as an enum member
+        if type(node.func) is (ast.Attribute) and node.func.attr == 'getattr':
+            F = lambda f: GetAttr(self.result[node.args[0]], node.args[1].s)
         # default behavior
-        F = lambda f: ReturnType(f, [self.result[arg] for arg in node.args])
+        else:
+            F = lambda f: ReturnType(f,
+                                     [self.result[arg] for arg in node.args])
         # op is used to drop previous value there
         self.combine(node, node.func, op=lambda x, y: y, unary_op=F)
 
@@ -642,51 +666,60 @@ class Types(ModuleAnalysis):
                 return w[n.id], (n.id,)
             elif isinstance(n, ast.Attribute):
                 r = rec(w, n.value)
+                if len(r[1]) > 1:
+                    plast, last = r[1][-2:]
+                    if plast == '__builtin__' and last.startswith('__'):
+                        return r[0][n.attr], r[1][:-2] + r[1][-1:] + (n.attr,)
                 return r[0][n.attr], r[1] + (n.attr,)
         obj, path = rec(modules, node)
-        self.result[node] = DeclType(
-                '::'.join(path) if obj.isliteral() else
-                ('::'.join(path[:-1]) + '::proxy::' + path[-1] + '()')
-                )
+        path = ('pythonic',) + path
+        if obj.isliteral():
+            self.result[node] = DeclType('::'.join(path))
+        else:
+            self.result[node] = DeclType(
+                '::'.join(path[:-1]) + '::proxy::' + path[-1] + '()')
 
     def visit_Slice(self, node):
         self.generic_visit(node)
-        self.result[node] = NamedType('core::slice')
+        if node.step is None or (type(node.step) is ast.Num
+                                 and node.step.n == 1):
+            self.result[node] = NamedType('pythonic::types::contiguous_slice')
+        else:
+            self.result[node] = NamedType('pythonic::types::slice')
 
     def visit_Subscript(self, node):
         self.visit(node.value)
-        if metadata.get(node, metadata.Attribute):
-            f = lambda t: AttributeType(node.slice.value.n, t)
-        elif isinstance(node.slice, ast.ExtSlice):
-            d = sum(int(type(dim) is ast.Index) for dim in node.slice.dims)
-            f = lambda t: reduce(lambda x, y: ContentType(x), range(d), t)
-        elif isinstance(node.slice, ast.Slice):
+        # type of a[1:2, 3, 4:1] is the type of: declval(a)(slice, long, slice)
+        if isinstance(node.slice, ast.ExtSlice):
             self.visit(node.slice)
-            f = lambda x:  ExpressionType(
-                    lambda a, b: "{0}[{1}]".format(a, b),
-                    [x, self.result[node.slice]]
-                    )
-        elif isinstance(node.slice.value, ast.Num) and node.slice.value.n >= 0:
+            f = lambda t: ExpressionType(
+                lambda a, *b: "{0}({1})".format(a, ", ".join(b)),
+                [t] + [self.result[d] for d in node.slice.dims]
+                )
+        elif (type(node.slice) is (ast.Index)
+                and type(node.slice.value) is ast.Num
+                and node.slice.value.n >= 0):
+            # type of a[2] is the type of an elements of a
+            # this special case is to make type inference easier
+            # for the back end compiler
             f = lambda t: ElementType(node.slice.value.n, t)
-        elif isinstance(node.slice.value, ast.Tuple):
-            f = lambda t: reduce(lambda x, y: ContentType(x),
-                    node.slice.value.elts, t)
         else:
+            # type of a[i] is the return type of the matching function
             self.visit(node.slice)
             f = lambda x: ExpressionType(
-                    lambda a, b: "{0}[{1}]".format(a, b),
-                    [x, self.result[node.slice]]
-                    )
+                lambda a, b: "{0}[{1}]".format(a, b),
+                [x, self.result[node.slice]]
+                )
         f and self.combine(node, node.value, unary_op=f)
 
     def visit_AssignedSubscript(self, node):
         if type(node.slice) not in (ast.Slice, ast.ExtSlice):
             self.visit(node.slice)
             self.combine(node.value, node.slice,
-                    unary_op=IndexableType, register=True)
+                         unary_op=IndexableType, register=True)
             for alias in self.strict_aliases[node.value].aliases:
                 self.combine(alias, node.slice,
-                        unary_op=IndexableType, register=True)
+                             unary_op=IndexableType, register=True)
             return True
         else:
             return False
@@ -706,7 +739,7 @@ class Types(ModuleAnalysis):
             for elt in node.elts:
                 self.combine(node, elt, unary_op=ListType)
         else:
-            self.result[node] = NamedType("core::empty_list")
+            self.result[node] = NamedType("pythonic::types::empty_list")
 
     def visit_Set(self, node):
         self.generic_visit(node)
@@ -714,21 +747,23 @@ class Types(ModuleAnalysis):
             for elt in node.elts:
                 self.combine(node, elt, unary_op=SetType)
         else:
-            self.result[node] = NamedType("core::empty_set")
+            self.result[node] = NamedType("pythonic::types::empty_set")
 
     def visit_Dict(self, node):
         self.generic_visit(node)
         if node.keys:
             for key, value in zip(node.keys, node.values):
                 self.combine(node, key,
-                        unary_op=lambda x: DictType(x, self.result[value]))
+                             unary_op=lambda x: DictType(x,
+                                                         self.result[value]))
         else:
-            self.result[node] = NamedType("core::empty_dict")
+            self.result[node] = NamedType("pythonic::types::empty_dict")
 
     def visit_ExceptHandler(self, node):
         if node.type and node.name:
             if not isinstance(node.type, ast.Tuple):
-                tname = NamedType('core::{0}'.format(node.type.attr))
+                tname = NamedType(
+                    'pythonic::types::{0}'.format(node.type.attr))
                 self.result[node.type] = tname
                 self.combine(node.name, node.type, register=True)
         map(self.visit, node.body)
