@@ -10,6 +10,8 @@ import sys
 import ast
 import cStringIO
 import os
+import metadata
+import openmp
 
 
 # Large float and imaginary literals get turned into infinities in the AST.
@@ -67,6 +69,18 @@ class Unparser:
 
     def dispatch(self, tree):
         "Dispatcher function, dispatching tree type T to method _T."
+        #display omp directive in python dump
+        for omp in metadata.get(tree, openmp.OMPDirective):
+            deps = list()
+            for dep in omp.deps:
+                old_file = self.f
+                self.f = cStringIO.StringIO()
+                self.dispatch(dep)
+                deps.append(self.f.getvalue())
+                self.f = old_file
+            directive = omp.s.format(*deps)
+            self._Expr(ast.Expr(ast.Str(s=directive)))
+
         if isinstance(tree, list):
             for t in tree:
                 self.dispatch(t)
@@ -82,8 +96,18 @@ class Unparser:
     ########################################################
 
     def _Module(self, tree):
+        # Goes through each top-level statement. If the special __init__()
+        # function is found, add a call to it because it's a special Pythran
+        # feature.
+        has_init = False
         for stmt in tree.body:
             self.dispatch(stmt)
+            if (type(stmt) is ast.FunctionDef and
+                    stmt.name == '__init__'):
+                has_init = True
+        # Call __init__() in which top statements are moved.
+        if has_init:
+            self.fill("__init__()")
 
     # stmt
     def _Expr(self, tree):
@@ -451,8 +475,8 @@ class Unparser:
         self.write(")")
 
     binop = {"Add": "+", "Sub": "-", "Mult": "*", "Div": "/", "Mod": "%",
-                "LShift": "<<", "RShift": ">>", "BitOr": "|", "BitXor": "^",
-                "BitAnd": "&", "FloorDiv": "//", "Pow": "**"}
+             "LShift": "<<", "RShift": ">>", "BitOr": "|", "BitXor": "^",
+             "BitAnd": "&", "FloorDiv": "//", "Pow": "**"}
 
     def _BinOp(self, t):
         self.write("(")
@@ -462,8 +486,8 @@ class Unparser:
         self.write(")")
 
     cmpops = {"Eq": "==", "NotEq": "!=", "Lt": "<", "LtE": "<=", "Gt": ">",
-                "GtE": ">=", "Is": "is", "IsNot": "is not", "In": "in",
-                "NotIn": "not in"}
+              "GtE": ">=", "Is": "is", "IsNot": "is not", "In": "in",
+              "NotIn": "not in"}
 
     def _Compare(self, t):
         self.write("(")
